/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.sourcery.resource.impl;

import biz.massivedynamics.sourcery.resource.Resource;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;

/**
 * An example implementation of Resource called Settings
 * 
 * Can be used in a production environment to handle settings, along with default values
 *
 * @author Cruz Bishop
 * @version 1.1.0.1
 */
public class Settings implements Resource<HashMap<String, Object>> {
    
    /**
     * A concurrent hash map of raw settings
     * 
     * @since 1.0.0.0
     */
    private ConcurrentHashMap<String, Object> settings = new ConcurrentHashMap<String, Object>();
    
    /**
     * Adds a setting
     * 
     * @param name The setting's name
     * @param setting The setting
     * @since 1.0.0.0
     */
    public void addSetting(String name, Object setting) {
        //Put the value as the permanent setting
        settings.put(name, setting);
    }
    
    /**
     * Removes a setting
     * 
     * @param name The setting's name
     * @since 1.1.0.0
     */
    public void removeSetting(String name) {
        settings.remove(name);
    }
    
    /**
     * Gets a setting, providing a default value if no value is found
     * 
     * @param name The setting name
     * @param _default The default setting value
     * @return The setting
     * @since 1.0.0.0
     */
    public Object getSetting(String name, Object _default) {
        //Get the setting, adding the default value if it's not found
        return this.settings.putIfAbsent(name, _default);
    }
    
    /**
     * Gets a setting without any default value
     * 
     * @param name The setting name
     * @return The value
     * @since 1.1.0.0
     */
    public Object getSetting(String name) {
        return settings.get(name);
    }

    /**
     * Loads the raw settings
     * 
     * @param raw The raw settings to load
     * @since 1.0.0.0
     */
    public void loadRaw(HashMap<String, Object> raw) {
        //Set the settings
        this.settings = new ConcurrentHashMap<String, Object>(raw);
    }

    /**
     * Gets the raw settings
     * 
     * @return A copy of the raw settings
     * @since 1.0.0.0
     */
    public HashMap<String, Object> getRaw() {
        return new HashMap<String, Object>(this.settings);
    }
    
}
